# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/adt_settings.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWidgetAdtSettings(object):
    def setupUi(self, MainWidgetAdtSettings):
        MainWidgetAdtSettings.setObjectName("MainWidgetAdtSettings")
        MainWidgetAdtSettings.resize(500, 700)
        self.gridLayout = QtWidgets.QGridLayout(MainWidgetAdtSettings)
        self.gridLayout.setObjectName("gridLayout")
        self.label_termNum = QtWidgets.QLabel(MainWidgetAdtSettings)
        self.label_termNum.setObjectName("label_termNum")
        self.gridLayout.addWidget(self.label_termNum, 0, 0, 1, 1)
        self.pushButton_termNumApply = QtWidgets.QPushButton(MainWidgetAdtSettings)
        self.pushButton_termNumApply.setObjectName("pushButton_termNumApply")
        self.gridLayout.addWidget(self.pushButton_termNumApply, 0, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 1, 1, 1, 1)
        self.comboBox_termNum = QtWidgets.QComboBox(MainWidgetAdtSettings)
        self.comboBox_termNum.setObjectName("comboBox_termNum")
        self.comboBox_termNum.addItem("")
        self.comboBox_termNum.addItem("")
        self.gridLayout.addWidget(self.comboBox_termNum, 0, 1, 1, 1)

        self.retranslateUi(MainWidgetAdtSettings)
        QtCore.QMetaObject.connectSlotsByName(MainWidgetAdtSettings)

    def retranslateUi(self, MainWidgetAdtSettings):
        _translate = QtCore.QCoreApplication.translate
        self.label_termNum.setText(_translate("MainWidgetAdtSettings", "Terminal 數量："))
        self.pushButton_termNumApply.setText(_translate("MainWidgetAdtSettings", "應用"))
        self.comboBox_termNum.setItemText(0, _translate("MainWidgetAdtSettings", "1"))
        self.comboBox_termNum.setItemText(1, _translate("MainWidgetAdtSettings", "2"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/hmi_load_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_HmiLoadDialog(object):
    def setupUi(self, HmiLoadDialog):
        HmiLoadDialog.setObjectName("HmiLoadDialog")
        HmiLoadDialog.resize(630, 350)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        HmiLoadDialog.setFont(font)
        HmiLoadDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(HmiLoadDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtWidgets.QTabWidget(HmiLoadDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_txt = QtWidgets.QWidget()
        self.tab_txt.setObjectName("tab_txt")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tab_txt)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton_txtLoad = QtWidgets.QPushButton(self.tab_txt)
        self.pushButton_txtLoad.setObjectName("pushButton_txtLoad")
        self.horizontalLayout.addWidget(self.pushButton_txtLoad)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.tabWidget.addTab(self.tab_txt, "")
        self.tab_mat = QtWidgets.QWidget()
        self.tab_mat.setObjectName("tab_mat")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_mat)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem2, 8, 1, 1, 1)
        self.pushButton_up = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_up.setObjectName("pushButton_up")
        self.gridLayout_3.addWidget(self.pushButton_up, 5, 1, 1, 1)
        self.pushButton_confirm = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_confirm.setObjectName("pushButton_confirm")
        self.gridLayout_3.addWidget(self.pushButton_confirm, 9, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem3, 2, 1, 1, 1)
        self.tableWidget_mat = QtWidgets.QTableWidget(self.tab_mat)
        self.tableWidget_mat.setObjectName("tableWidget_mat")
        self.tableWidget_mat.setColumnCount(5)
        self.tableWidget_mat.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(4, item)
        self.gridLayout_3.addWidget(self.tableWidget_mat, 1, 0, 9, 1)
        self.pushButton_down = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_down.setObjectName("pushButton_down")
        self.gridLayout_3.addWidget(self.pushButton_down, 6, 1, 1, 1)
        self.pushButton_delete = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_delete.setObjectName("pushButton_delete")
        self.gridLayout_3.addWidget(self.pushButton_delete, 7, 1, 1, 1)
        self.pushButton_matLoad = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_matLoad.setObjectName("pushButton_matLoad")
        self.gridLayout_3.addWidget(self.pushButton_matLoad, 1, 1, 1, 1)
        self.pushButton_newSeq = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_newSeq.setObjectName("pushButton_newSeq")
        self.gridLayout_3.addWidget(self.pushButton_newSeq, 3, 1, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem4, 4, 1, 1, 1)
        self.tabWidget.addTab(self.tab_mat, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)

        self.retranslateUi(HmiLoadDialog)
        self.tabWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(HmiLoadDialog)
        HmiLoadDialog.setTabOrder(self.tabWidget, self.tableWidget_mat)
        HmiLoadDialog.setTabOrder(self.tableWidget_mat, self.pushButton_matLoad)
        HmiLoadDialog.setTabOrder(self.pushButton_matLoad, self.pushButton_newSeq)
        HmiLoadDialog.setTabOrder(self.pushButton_newSeq, self.pushButton_up)
        HmiLoadDialog.setTabOrder(self.pushButton_up, self.pushButton_down)
        HmiLoadDialog.setTabOrder(self.pushButton_down, self.pushButton_delete)
        HmiLoadDialog.setTabOrder(self.pushButton_delete, self.pushButton_confirm)

    def retranslateUi(self, HmiLoadDialog):
        _translate = QtCore.QCoreApplication.translate
        HmiLoadDialog.setWindowTitle(_translate("HmiLoadDialog", "HmiLoadDialog"))
        self.pushButton_txtLoad.setText(_translate("HmiLoadDialog", "選擇檔案"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_txt), _translate("HmiLoadDialog", "txt檔案"))
        self.pushButton_up.setText(_translate("HmiLoadDialog", "上移"))
        self.pushButton_confirm.setText(_translate("HmiLoadDialog", "確認"))
        item = self.tableWidget_mat.horizontalHeaderItem(0)
        item.setText(_translate("HmiLoadDialog", "新順序"))
        item = self.tableWidget_mat.horizontalHeaderItem(1)
        item.setText(_translate("HmiLoadDialog", "變數名稱"))
        item = self.tableWidget_mat.horizontalHeaderItem(2)
        item.setText(_translate("HmiLoadDialog", "資料型態"))
        item = self.tableWidget_mat.horizontalHeaderItem(3)
        item.setText(_translate("HmiLoadDialog", "個數"))
        item = self.tableWidget_mat.horizontalHeaderItem(4)
        item.setText(_translate("HmiLoadDialog", "總資料大小(byte)"))
        self.pushButton_down.setText(_translate("HmiLoadDialog", "下移"))
        self.pushButton_delete.setText(_translate("HmiLoadDialog", "刪除"))
        self.pushButton_matLoad.setText(_translate("HmiLoadDialog", "選擇檔案"))
        self.pushButton_newSeq.setText(_translate("HmiLoadDialog", "應用新順序"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_mat), _translate("HmiLoadDialog", "mat file"))




from asa_hmi_data_agent.cli_tools.commands import *
from asa_hmi_data_agent.cli_tools.socket_handler import SocketHandler

import argparse
import progressbar

def argHandler():
    parser = argparse.ArgumentParser(description='tell adt to load hex into asa-series board')
    parser.add_argument('-H', '--hex',
                        dest='hexfile', action ='store', type = str,
                        help='assign hex file to be load')
    parser.add_argument('-P', '-p', '--port',
                        dest='port', action ='store', type = str,
                        help='assign the port to load')
    parser.add_argument('-s', '-S', '--set',
                        dest='set', action ='store', type = int,
                        default=0,
                        help='use existing set (1~N)')
    args = parser.parse_args()
    return args

def run():
    args = argHandler()
    print(args)
    startCmd = {
        'cmd': AdtCmd.LOADER.value,
        'subcmd': AdtSubCmdLoader.START.value,
        'port': args.port,
        'hexfile': args.hexfile,
        'set': args.set
    }
    stateCmd = {
        'cmd' : AdtCmd.LOADER.value,
        'subcmd' : AdtSubCmdLoader.STATE.value,
    }

    adtsh = SocketHandler()
    res = adtsh.send_cmd(startCmd)
    print(res)
    if res is None:
        print('error!')
        return
    max = res['max']

    widgets=[
    ' [', progressbar.Timer(), '] ',
    progressbar.Bar(),
    progressbar.Counter(format='%(percentage)0.2f%%'),
    ]
    bar = progressbar.ProgressBar(max_value=max, widgets=widgets)

    times = 0
    while times != max:
        res = adtsh.send_cmd(stateCmd)
        print(res)
        if res is None:
            return
        times = res['times']
        bar.update(times)

if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/avrdude.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWidgetAvrdude(object):
    def setupUi(self, MainWidgetAvrdude):
        MainWidgetAvrdude.setObjectName("MainWidgetAvrdude")
        MainWidgetAvrdude.resize(500, 700)
        self.gridLayout_8 = QtWidgets.QGridLayout(MainWidgetAvrdude)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.groupBox_config = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_config.setFont(font)
        self.groupBox_config.setObjectName("groupBox_config")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox_config)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.comboBox_config = QtWidgets.QComboBox(self.groupBox_config)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.comboBox_config.setFont(font)
        self.comboBox_config.setEditable(True)
        self.comboBox_config.setObjectName("comboBox_config")
        self.comboBox_config.addItem("")
        self.horizontalLayout.addWidget(self.comboBox_config)
        self.pushButton_configSave = QtWidgets.QPushButton(self.groupBox_config)
        self.pushButton_configSave.setMaximumSize(QtCore.QSize(100, 16777215))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_configSave.setFont(font)
        self.pushButton_configSave.setObjectName("pushButton_configSave")
        self.horizontalLayout.addWidget(self.pushButton_configSave)
        self.pushButton_configDelete = QtWidgets.QPushButton(self.groupBox_config)
        self.pushButton_configDelete.setMaximumSize(QtCore.QSize(100, 16777215))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_configDelete.setFont(font)
        self.pushButton_configDelete.setObjectName("pushButton_configDelete")
        self.horizontalLayout.addWidget(self.pushButton_configDelete)
        self.gridLayout_8.addWidget(self.groupBox_config, 0, 0, 1, 4)
        self.groupBox_serial = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_serial.setFont(font)
        self.groupBox_serial.setObjectName("groupBox_serial")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_serial)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_serialSetPort = QtWidgets.QLabel(self.groupBox_serial)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_serialSetPort.setFont(font)
        self.label_serialSetPort.setObjectName("label_serialSetPort")
        self.gridLayout_7.addWidget(self.label_serialSetPort, 0, 1, 1, 1)
        self.label_serialSetBaud = QtWidgets.QLabel(self.groupBox_serial)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_serialSetBaud.setFont(font)
        self.label_serialSetBaud.setObjectName("label_serialSetBaud")
        self.gridLayout_7.addWidget(self.label_serialSetBaud, 0, 2, 1, 1)
        self.lineEdit_serialSetBaud = QtWidgets.QLineEdit(self.groupBox_serial)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_serialSetBaud.setFont(font)
        self.lineEdit_serialSetBaud.setObjectName("lineEdit_serialSetBaud")
        self.gridLayout_7.addWidget(self.lineEdit_serialSetBaud, 1, 2, 1, 1)
        self.comboBox_serialSetPort = QtWidgets.QComboBox(self.groupBox_serial)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.comboBox_serialSetPort.setFont(font)
        self.comboBox_serialSetPort.setObjectName("comboBox_serialSetPort")
        self.gridLayout_7.addWidget(self.comboBox_serialSetPort, 1, 1, 1, 1)
        self.pushButton_updatePort = QtWidgets.QPushButton(self.groupBox_serial)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.pushButton_updatePort.setFont(font)
        self.pushButton_updatePort.setObjectName("pushButton_updatePort")
        self.gridLayout_7.addWidget(self.pushButton_updatePort, 0, 0, 2, 1)
        self.gridLayout_8.addWidget(self.groupBox_serial, 1, 0, 1, 3)
        self.groupBox_flash = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_flash.setFont(font)
        self.groupBox_flash.setObjectName("groupBox_flash")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_flash)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lineEdit_flash = QtWidgets.QLineEdit(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_flash.setFont(font)
        self.lineEdit_flash.setObjectName("lineEdit_flash")
        self.gridLayout_2.addWidget(self.lineEdit_flash, 0, 0, 1, 4)
        self.toolButton_flash = QtWidgets.QToolButton(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.toolButton_flash.setFont(font)
        self.toolButton_flash.setObjectName("toolButton_flash")
        self.gridLayout_2.addWidget(self.toolButton_flash, 0, 4, 1, 1)
        self.radioButton_flashRead = QtWidgets.QRadioButton(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_flashRead.setFont(font)
        self.radioButton_flashRead.setAutoExclusive(False)
        self.radioButton_flashRead.setObjectName("radioButton_flashRead")
        self.gridLayout_2.addWidget(self.radioButton_flashRead, 1, 1, 1, 1)
        self.radioButton_flashWrite = QtWidgets.QRadioButton(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_flashWrite.setFont(font)
        self.radioButton_flashWrite.setAutoExclusive(False)
        self.radioButton_flashWrite.setObjectName("radioButton_flashWrite")
        self.gridLayout_2.addWidget(self.radioButton_flashWrite, 1, 0, 1, 1)
        self.radioButton_flashVerify = QtWidgets.QRadioButton(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_flashVerify.setFont(font)
        self.radioButton_flashVerify.setAutoExclusive(False)
        self.radioButton_flashVerify.setObjectName("radioButton_flashVerify")
        self.gridLayout_2.addWidget(self.radioButton_flashVerify, 1, 2, 1, 1)
        self.pushButton_flashGo = QtWidgets.QPushButton(self.groupBox_flash)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_flashGo.setFont(font)
        self.pushButton_flashGo.setObjectName("pushButton_flashGo")
        self.gridLayout_2.addWidget(self.pushButton_flashGo, 1, 3, 1, 2)
        self.gridLayout_8.addWidget(self.groupBox_flash, 2, 0, 2, 3)
        self.groupBox_eeprom = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_eeprom.setFont(font)
        self.groupBox_eeprom.setObjectName("groupBox_eeprom")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_eeprom)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.lineEdit_eeprom = QtWidgets.QLineEdit(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_eeprom.setFont(font)
        self.lineEdit_eeprom.setObjectName("lineEdit_eeprom")
        self.gridLayout_3.addWidget(self.lineEdit_eeprom, 0, 0, 1, 4)
        self.toolButton_eeprom = QtWidgets.QToolButton(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.toolButton_eeprom.setFont(font)
        self.toolButton_eeprom.setObjectName("toolButton_eeprom")
        self.gridLayout_3.addWidget(self.toolButton_eeprom, 0, 4, 1, 1)
        self.radioButton_eepromRead = QtWidgets.QRadioButton(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_eepromRead.setFont(font)
        self.radioButton_eepromRead.setAutoExclusive(False)
        self.radioButton_eepromRead.setObjectName("radioButton_eepromRead")
        self.gridLayout_3.addWidget(self.radioButton_eepromRead, 1, 1, 1, 1)
        self.radioButton_eepromWrite = QtWidgets.QRadioButton(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_eepromWrite.setFont(font)
        self.radioButton_eepromWrite.setAutoExclusive(False)
        self.radioButton_eepromWrite.setObjectName("radioButton_eepromWrite")
        self.gridLayout_3.addWidget(self.radioButton_eepromWrite, 1, 0, 1, 1)
        self.radioButton_eepromVerify = QtWidgets.QRadioButton(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.radioButton_eepromVerify.setFont(font)
        self.radioButton_eepromVerify.setAutoExclusive(False)
        self.radioButton_eepromVerify.setObjectName("radioButton_eepromVerify")
        self.gridLayout_3.addWidget(self.radioButton_eepromVerify, 1, 2, 1, 1)
        self.pushButton_eepromGo = QtWidgets.QPushButton(self.groupBox_eeprom)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_eepromGo.setFont(font)
        self.pushButton_eepromGo.setObjectName("pushButton_eepromGo")
        self.gridLayout_3.addWidget(self.pushButton_eepromGo, 1, 3, 1, 2)
        self.gridLayout_8.addWidget(self.groupBox_eeprom, 4, 0, 1, 3)
        self.groupBox_else = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_else.setFont(font)
        self.groupBox_else.setObjectName("groupBox_else")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_else)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.checkBox_cancelVerify = QtWidgets.QCheckBox(self.groupBox_else)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.checkBox_cancelVerify.setFont(font)
        self.checkBox_cancelVerify.setObjectName("checkBox_cancelVerify")
        self.gridLayout_6.addWidget(self.checkBox_cancelVerify, 0, 0, 1, 1)
        self.checkBox_eraseChip = QtWidgets.QCheckBox(self.groupBox_else)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.checkBox_eraseChip.setFont(font)
        self.checkBox_eraseChip.setObjectName("checkBox_eraseChip")
        self.gridLayout_6.addWidget(self.checkBox_eraseChip, 0, 1, 1, 1)
        self.label_additionalParameter = QtWidgets.QLabel(self.groupBox_else)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.label_additionalParameter.setFont(font)
        self.label_additionalParameter.setObjectName("label_additionalParameter")
        self.gridLayout_6.addWidget(self.label_additionalParameter, 1, 0, 1, 1)
        self.lineEdit_additionalParameter = QtWidgets.QLineEdit(self.groupBox_else)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_additionalParameter.setFont(font)
        self.lineEdit_additionalParameter.setObjectName("lineEdit_additionalParameter")
        self.gridLayout_6.addWidget(self.lineEdit_additionalParameter, 1, 1, 1, 1)
        self.gridLayout_8.addWidget(self.groupBox_else, 5, 0, 2, 3)
        self.pushButton_startProgram = QtWidgets.QPushButton(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_startProgram.setFont(font)
        self.pushButton_startProgram.setObjectName("pushButton_startProgram")
        self.gridLayout_8.addWidget(self.pushButton_startProgram, 7, 0, 1, 1)
        self.pushButton_stopProgram = QtWidgets.QPushButton(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_stopProgram.setFont(font)
        self.pushButton_stopProgram.setObjectName("pushButton_stopProgram")
        self.gridLayout_8.addWidget(self.pushButton_stopProgram, 7, 1, 1, 1)
        self.pushButton_bitSelector = QtWidgets.QPushButton(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_bitSelector.setFont(font)
        self.pushButton_bitSelector.setObjectName("pushButton_bitSelector")
        self.gridLayout_8.addWidget(self.pushButton_bitSelector, 7, 3, 1, 1)
        self.textBrowser_cmd = QtWidgets.QTextBrowser(MainWidgetAvrdude)
        self.textBrowser_cmd.setMinimumSize(QtCore.QSize(0, 20))
        self.textBrowser_cmd.setMaximumSize(QtCore.QSize(16777215, 20))
        self.textBrowser_cmd.setSizeIncrement(QtCore.QSize(0, 20))
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(8)
        self.textBrowser_cmd.setFont(font)
        self.textBrowser_cmd.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.textBrowser_cmd.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textBrowser_cmd.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textBrowser_cmd.setObjectName("textBrowser_cmd")
        self.gridLayout_8.addWidget(self.textBrowser_cmd, 8, 0, 1, 4)
        self.textBrowser_cmdterminal = QtWidgets.QTextBrowser(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(8)
        self.textBrowser_cmdterminal.setFont(font)
        self.textBrowser_cmdterminal.setObjectName("textBrowser_cmdterminal")
        self.gridLayout_8.addWidget(self.textBrowser_cmdterminal, 9, 0, 1, 4)
        self.groupBox_mcu = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_mcu.setFont(font)
        self.groupBox_mcu.setObjectName("groupBox_mcu")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_mcu)
        self.gridLayout.setObjectName("gridLayout")
        self.comboBox_mcuSelect = QtWidgets.QComboBox(self.groupBox_mcu)
        self.comboBox_mcuSelect.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.comboBox_mcuSelect.setFont(font)
        self.comboBox_mcuSelect.setObjectName("comboBox_mcuSelect")
        self.comboBox_mcuSelect.addItem("")
        self.comboBox_mcuSelect.addItem("")
        self.gridLayout.addWidget(self.comboBox_mcuSelect, 0, 0, 1, 2)
        self.pushButton_mcuDetect = QtWidgets.QPushButton(self.groupBox_mcu)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_mcuDetect.setFont(font)
        self.pushButton_mcuDetect.setObjectName("pushButton_mcuDetect")
        self.gridLayout.addWidget(self.pushButton_mcuDetect, 1, 0, 1, 2)
        self.gridLayout_8.addWidget(self.groupBox_mcu, 1, 3, 1, 1)
        self.groupBox_lock = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_lock.setFont(font)
        self.groupBox_lock.setObjectName("groupBox_lock")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_lock)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_lockTitle = QtWidgets.QLabel(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_lockTitle.setFont(font)
        self.label_lockTitle.setObjectName("label_lockTitle")
        self.gridLayout_5.addWidget(self.label_lockTitle, 0, 0, 1, 1)
        self.lineEdit_lock = QtWidgets.QLineEdit(self.groupBox_lock)
        self.lineEdit_lock.setMinimumSize(QtCore.QSize(40, 0))
        self.lineEdit_lock.setMaximumSize(QtCore.QSize(50, 16777215))
        self.lineEdit_lock.setBaseSize(QtCore.QSize(50, 0))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_lock.setFont(font)
        self.lineEdit_lock.setObjectName("lineEdit_lock")
        self.gridLayout_5.addWidget(self.lineEdit_lock, 0, 1, 1, 1)
        self.pushButton_lockWrite = QtWidgets.QPushButton(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_lockWrite.setFont(font)
        self.pushButton_lockWrite.setObjectName("pushButton_lockWrite")
        self.gridLayout_5.addWidget(self.pushButton_lockWrite, 1, 2, 1, 1)
        self.pushButton_lockRead = QtWidgets.QPushButton(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_lockRead.setFont(font)
        self.pushButton_lockRead.setObjectName("pushButton_lockRead")
        self.gridLayout_5.addWidget(self.pushButton_lockRead, 0, 2, 1, 1)
        self.checkBox_lockSet = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.checkBox_lockSet.setFont(font)
        self.checkBox_lockSet.setObjectName("checkBox_lockSet")
        self.gridLayout_5.addWidget(self.checkBox_lockSet, 1, 0, 1, 2)
        self.gridLayout_8.addWidget(self.groupBox_lock, 5, 3, 2, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(MainWidgetAvrdude)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.groupBox_5.setFont(font)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.pushButton_fuseRead = QtWidgets.QPushButton(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_fuseRead.setFont(font)
        self.pushButton_fuseRead.setObjectName("pushButton_fuseRead")
        self.gridLayout_4.addWidget(self.pushButton_fuseRead, 0, 2, 1, 1)
        self.lineEdit_fuseHigh = QtWidgets.QLineEdit(self.groupBox_5)
        self.lineEdit_fuseHigh.setMinimumSize(QtCore.QSize(40, 0))
        self.lineEdit_fuseHigh.setMaximumSize(QtCore.QSize(50, 16777215))
        self.lineEdit_fuseHigh.setBaseSize(QtCore.QSize(50, 0))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_fuseHigh.setFont(font)
        self.lineEdit_fuseHigh.setObjectName("lineEdit_fuseHigh")
        self.gridLayout_4.addWidget(self.lineEdit_fuseHigh, 1, 1, 1, 1)
        self.label_fuseHigh = QtWidgets.QLabel(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_fuseHigh.setFont(font)
        self.label_fuseHigh.setObjectName("label_fuseHigh")
        self.gridLayout_4.addWidget(self.label_fuseHigh, 1, 0, 1, 1)
        self.pushButton_fuseWrite = QtWidgets.QPushButton(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.pushButton_fuseWrite.setFont(font)
        self.pushButton_fuseWrite.setObjectName("pushButton_fuseWrite")
        self.gridLayout_4.addWidget(self.pushButton_fuseWrite, 1, 2, 1, 1)
        self.label_fuseExtra = QtWidgets.QLabel(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_fuseExtra.setFont(font)
        self.label_fuseExtra.setObjectName("label_fuseExtra")
        self.gridLayout_4.addWidget(self.label_fuseExtra, 2, 0, 1, 1)
        self.lineEdit_fuseExtra = QtWidgets.QLineEdit(self.groupBox_5)
        self.lineEdit_fuseExtra.setMinimumSize(QtCore.QSize(40, 0))
        self.lineEdit_fuseExtra.setMaximumSize(QtCore.QSize(50, 16777215))
        self.lineEdit_fuseExtra.setBaseSize(QtCore.QSize(50, 0))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_fuseExtra.setFont(font)
        self.lineEdit_fuseExtra.setObjectName("lineEdit_fuseExtra")
        self.gridLayout_4.addWidget(self.lineEdit_fuseExtra, 2, 1, 1, 1)
        self.label_fuseLow = QtWidgets.QLabel(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.label_fuseLow.setFont(font)
        self.label_fuseLow.setObjectName("label_fuseLow")
        self.gridLayout_4.addWidget(self.label_fuseLow, 0, 0, 1, 1)
        self.lineEdit_fuseLow = QtWidgets.QLineEdit(self.groupBox_5)
        self.lineEdit_fuseLow.setMinimumSize(QtCore.QSize(40, 0))
        self.lineEdit_fuseLow.setMaximumSize(QtCore.QSize(50, 16777215))
        self.lineEdit_fuseLow.setBaseSize(QtCore.QSize(50, 0))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.lineEdit_fuseLow.setFont(font)
        self.lineEdit_fuseLow.setObjectName("lineEdit_fuseLow")
        self.gridLayout_4.addWidget(self.lineEdit_fuseLow, 0, 1, 1, 1)
        self.checkBox_fuseSet = QtWidgets.QCheckBox(self.groupBox_5)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.checkBox_fuseSet.setFont(font)
        self.checkBox_fuseSet.setObjectName("checkBox_fuseSet")
        self.gridLayout_4.addWidget(self.checkBox_fuseSet, 3, 0, 1, 2)
        self.gridLayout_8.addWidget(self.groupBox_5, 2, 3, 3, 1)

        self.retranslateUi(MainWidgetAvrdude)
        QtCore.QMetaObject.connectSlotsByName(MainWidgetAvrdude)
        MainWidgetAvrdude.setTabOrder(self.comboBox_config, self.pushButton_configSave)
        MainWidgetAvrdude.setTabOrder(self.pushButton_configSave, self.pushButton_configDelete)
        MainWidgetAvrdude.setTabOrder(self.pushButton_configDelete, self.pushButton_updatePort)
        MainWidgetAvrdude.setTabOrder(self.pushButton_updatePort, self.comboBox_serialSetPort)
        MainWidgetAvrdude.setTabOrder(self.comboBox_serialSetPort, self.lineEdit_serialSetBaud)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_serialSetBaud, self.comboBox_mcuSelect)
        MainWidgetAvrdude.setTabOrder(self.comboBox_mcuSelect, self.pushButton_mcuDetect)
        MainWidgetAvrdude.setTabOrder(self.pushButton_mcuDetect, self.lineEdit_flash)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_flash, self.toolButton_flash)
        MainWidgetAvrdude.setTabOrder(self.toolButton_flash, self.radioButton_flashWrite)
        MainWidgetAvrdude.setTabOrder(self.radioButton_flashWrite, self.radioButton_flashRead)
        MainWidgetAvrdude.setTabOrder(self.radioButton_flashRead, self.radioButton_flashVerify)
        MainWidgetAvrdude.setTabOrder(self.radioButton_flashVerify, self.pushButton_flashGo)
        MainWidgetAvrdude.setTabOrder(self.pushButton_flashGo, self.lineEdit_eeprom)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_eeprom, self.toolButton_eeprom)
        MainWidgetAvrdude.setTabOrder(self.toolButton_eeprom, self.radioButton_eepromWrite)
        MainWidgetAvrdude.setTabOrder(self.radioButton_eepromWrite, self.radioButton_eepromRead)
        MainWidgetAvrdude.setTabOrder(self.radioButton_eepromRead, self.radioButton_eepromVerify)
        MainWidgetAvrdude.setTabOrder(self.radioButton_eepromVerify, self.pushButton_eepromGo)
        MainWidgetAvrdude.setTabOrder(self.pushButton_eepromGo, self.lineEdit_fuseLow)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_fuseLow, self.pushButton_fuseRead)
        MainWidgetAvrdude.setTabOrder(self.pushButton_fuseRead, self.lineEdit_fuseHigh)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_fuseHigh, self.pushButton_fuseWrite)
        MainWidgetAvrdude.setTabOrder(self.pushButton_fuseWrite, self.lineEdit_fuseExtra)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_fuseExtra, self.checkBox_fuseSet)
        MainWidgetAvrdude.setTabOrder(self.checkBox_fuseSet, self.checkBox_cancelVerify)
        MainWidgetAvrdude.setTabOrder(self.checkBox_cancelVerify, self.checkBox_eraseChip)
        MainWidgetAvrdude.setTabOrder(self.checkBox_eraseChip, self.lineEdit_additionalParameter)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_additionalParameter, self.lineEdit_lock)
        MainWidgetAvrdude.setTabOrder(self.lineEdit_lock, self.pushButton_lockRead)
        MainWidgetAvrdude.setTabOrder(self.pushButton_lockRead, self.checkBox_lockSet)
        MainWidgetAvrdude.setTabOrder(self.checkBox_lockSet, self.pushButton_lockWrite)
        MainWidgetAvrdude.setTabOrder(self.pushButton_lockWrite, self.pushButton_startProgram)
        MainWidgetAvrdude.setTabOrder(self.pushButton_startProgram, self.pushButton_stopProgram)
        MainWidgetAvrdude.setTabOrder(self.pushButton_stopProgram, self.pushButton_bitSelector)
        MainWidgetAvrdude.setTabOrder(self.pushButton_bitSelector, self.textBrowser_cmd)
        MainWidgetAvrdude.setTabOrder(self.textBrowser_cmd, self.textBrowser_cmdterminal)

    def retranslateUi(self, MainWidgetAvrdude):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox_config.setTitle(_translate("MainWidgetAvrdude", "設定檔"))
        self.comboBox_config.setItemText(0, _translate("MainWidgetAvrdude", "選擇設定檔..."))
        self.pushButton_configSave.setText(_translate("MainWidgetAvrdude", "儲存"))
        self.pushButton_configDelete.setText(_translate("MainWidgetAvrdude", "刪除"))
        self.groupBox_serial.setTitle(_translate("MainWidgetAvrdude", "燒錄器設定"))
        self.label_serialSetPort.setText(_translate("MainWidgetAvrdude", "Port  (-P)"))
        self.label_serialSetBaud.setText(_translate("MainWidgetAvrdude", "Baud rate (-b)"))
        self.pushButton_updatePort.setText(_translate("MainWidgetAvrdude", "更新串列埠"))
        self.groupBox_flash.setTitle(_translate("MainWidgetAvrdude", "Flash"))
        self.toolButton_flash.setText(_translate("MainWidgetAvrdude", "..."))
        self.radioButton_flashRead.setText(_translate("MainWidgetAvrdude", "Read"))
        self.radioButton_flashWrite.setText(_translate("MainWidgetAvrdude", "Write"))
        self.radioButton_flashVerify.setText(_translate("MainWidgetAvrdude", "Verify"))
        self.pushButton_flashGo.setText(_translate("MainWidgetAvrdude", "Go"))
        self.groupBox_eeprom.setTitle(_translate("MainWidgetAvrdude", "EEPROM"))
        self.toolButton_eeprom.setText(_translate("MainWidgetAvrdude", "..."))
        self.radioButton_eepromRead.setText(_translate("MainWidgetAvrdude", "Read"))
        self.radioButton_eepromWrite.setText(_translate("MainWidgetAvrdude", "Write"))
        self.radioButton_eepromVerify.setText(_translate("MainWidgetAvrdude", "Verify"))
        self.pushButton_eepromGo.setText(_translate("MainWidgetAvrdude", "Go"))
        self.groupBox_else.setTitle(_translate("MainWidgetAvrdude", "其餘選項"))
        self.checkBox_cancelVerify.setText(_translate("MainWidgetAvrdude", "取消驗證"))
        self.checkBox_eraseChip.setText(_translate("MainWidgetAvrdude", "清除晶片"))
        self.label_additionalParameter.setText(_translate("MainWidgetAvrdude", "增加參數"))
        self.pushButton_startProgram.setText(_translate("MainWidgetAvrdude", "START"))
        self.pushButton_stopProgram.setText(_translate("MainWidgetAvrdude", "強制終止"))
        self.pushButton_bitSelector.setText(_translate("MainWidgetAvrdude", "BitSelector"))
        self.groupBox_mcu.setTitle(_translate("MainWidgetAvrdude", "MCU"))
        self.comboBox_mcuSelect.setItemText(0, _translate("MainWidgetAvrdude", "請選擇MCU..."))
        self.comboBox_mcuSelect.setItemText(1, _translate("MainWidgetAvrdude", "ATmega88"))
        self.pushButton_mcuDetect.setText(_translate("MainWidgetAvrdude", "偵測"))
        self.groupBox_lock.setTitle(_translate("MainWidgetAvrdude", "Lock bits"))
        self.label_lockTitle.setText(_translate("MainWidgetAvrdude", "Lock"))
        self.pushButton_lockWrite.setText(_translate("MainWidgetAvrdude", "Write"))
        self.pushButton_lockRead.setText(_translate("MainWidgetAvrdude", "Read"))
        self.checkBox_lockSet.setText(_translate("MainWidgetAvrdude", "Set Lock bits"))
        self.groupBox_5.setTitle(_translate("MainWidgetAvrdude", "Fuse bits"))
        self.pushButton_fuseRead.setText(_translate("MainWidgetAvrdude", "Read"))
        self.label_fuseHigh.setText(_translate("MainWidgetAvrdude", "High"))
        self.pushButton_fuseWrite.setText(_translate("MainWidgetAvrdude", "Write"))
        self.label_fuseExtra.setText(_translate("MainWidgetAvrdude", "Extra"))
        self.label_fuseLow.setText(_translate("MainWidgetAvrdude", "Low"))
        self.checkBox_fuseSet.setText(_translate("MainWidgetAvrdude", "Set Fuse"))




from asa_hmi_data_agent.ui.ui_adt_settings import Ui_MainWidgetAdtSettings
from PyQt5.QtCore import pyqtSlot, QThread, pyqtSignal, QObject
from PyQt5.QtWidgets import QFileDialog
from asa_hmi_data_agent.listport import getAvailableSerialPorts
from time import sleep
import serial
import math

class AdtSettings(QObject):
    widget = Ui_MainWidgetAdtSettings()
    signalTermNumApply = pyqtSignal(int)

    def __init__(self):
        super(AdtSettings, self).__init__()

    def setupUi(self, widget):
        self.widget.setupUi(widget)

    def init(self):
        self.widget.pushButton_termNumApply.clicked.connect(self.termNumApply)

    def termNumApply(self):
        num = int(self.widget.comboBox_termNum.currentText())
        self.signalTermNumApply.emit(num)



import zmq
import json

class SocketHandler(object):
    def __init__(self):
        super(SocketHandler, self).__init__()
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)
        self.poll = zmq.Poller()
        self.poll.register(self.socket, zmq.POLLIN)

        self.socket.connect("tcp://127.0.0.1:8787")

    def close(self):
        self.socket.setsockopt(zmq.LINGER, 0)
        self.socket.close()
        self.poll.unregister(self.socket)

    def send_cmd(self, cmd):
        self.socket.send_json(json.dumps(cmd))
        socks = dict(self.poll.poll(3000))
        if socks.get(self.socket) == zmq.POLLIN:
            res = self.socket.recv_json()
            return json.loads(res)
        else:
            self.close()
            return None



import numpy as np
import scipy.io
import os.path
from PyQt5 import QtCore
from PyQt5.QtWidgets import QFileDialog, QDialog, QTableWidgetItem
from asa_hmi_data_agent.ui.ui_hmi_load_dialog import Ui_HmiLoadDialog
from .data_to_text import arToStr, stToStr
from ..hmipac.type import *

# Reference : https://docs.scipy.org/doc/scipy/reference/tutorial/io.html
# Reference : https://docs.scipy.org/doc/numpy-1.13.0/user/basics.types.html

class Const:
    COL_NEWSQE = 0
    COL_NAME = 1
    COL_TYPE = 2
    COL_NUMS = 3
    COL_SIZE = 4

# ---- class BitsSelector Start ------------------------------------------------
class HmiLoadDialog(QDialog, Ui_HmiLoadDialog):
    dataList = list()
    nameList = list()
    resText  = str()
    filename = str()

    def __init__(self):
        QDialog.__init__(self)
        self.setupUi(self)

        self.pushButton_txtLoad.clicked.connect(self.loadTxtFile)
        self.pushButton_matLoad.clicked.connect(self.loadMatFile)
        self.pushButton_confirm.clicked.connect(self.matConfirm)

        self.pushButton_up.clicked.connect(self.colMoveUp)
        self.pushButton_down.clicked.connect(self.colMoveDown)
        self.pushButton_delete.clicked.connect(self.colDelete)
        self.pushButton_newSeq.clicked.connect(self.applyNewSeq)

    def show(self):
        self.tableWidget_mat.setRowCount(0)
        self.dataList = list()
        self.resText  = ''
        super(QDialog, self).show()

    def matConfirm(self):
        self.updateTextFromDataList()
        self.accept()

    def loadTxtFile(self):
        # TODO load file
        filename, _ = QFileDialog.getOpenFileName(self, 'Open File','', 'All Files (*);;txt Files (*.txt);;' ,initialFilter='txt Files (*.txt)')
        if filename != '':
            with open(filename, 'r') as f:
                self.filename = filename
                self.resText = '// load from txt file: ' + filename + '\n\n'
                self.resText += f.read()
            self.accept()

    def loadMatFile(self):
        filename, _ = QFileDialog.getOpenFileName(self, 'Open File','', 'All Files (*);;Mat Files (*.mat);;' ,initialFilter='Mat Files (*.mat)')
        if filename == '':
            return
        else:
            matDict = scipy.io.loadmat(filename)
            self.filename = filename
            self.dataList = list()

        for key, val in matDict.items():
            if (
                    key == '__header__' or
                    key == '__version__' or
                    key == '__globals__'
                ):
                    pass
            else:
                self.dataList.append(val)
                self.nameList.append(key)
        for i in range(len(self.dataList)):
            if self.dataList[i].dtype.names is None:
                # array
                self.dataList[i] = self.dataList[i][0]
            else:
                # struct
                self.dataList[i] = reStructureData(self.dataList[i])
        self.updateTableFromDataList()

    def updateTableFromDataList(self):
        self.tableWidget_mat.clearContents()
        self.tableWidget_mat.setRowCount(len(self.dataList))
        for row in range(len(self.dataList)):
            data = self.dataList[row]
            nameStr = self.nameList[row]
            # NOTE the structured data of loadmat's return is strange.
            if data.shape is ():
                # Struct
                typeStr = getFs(data.dtype)
                numsStr = '1'
                sizeStr = str(data.dtype.itemsize)
            else:
                # Array
                typeStr = getTypeStr(getTypeNum(data.dtype.name))
                numsStr = str(data.size)
                sizeStr = str(data.size * data.dtype.itemsize)

            nameItem = QTableWidgetItem(nameStr)
            typeItem = QTableWidgetItem(typeStr)
            numsItem = QTableWidgetItem(numsStr)
            sizeItem = QTableWidgetItem(sizeStr)
            nameItem.setFlags(QtCore.Qt.ItemIsEnabled)
            typeItem.setFlags(QtCore.Qt.ItemIsEnabled)
            numsItem.setFlags(QtCore.Qt.ItemIsEnabled)
            sizeItem.setFlags(QtCore.Qt.ItemIsEnabled)
            self.tableWidget_mat.setItem(row, Const.COL_NAME, nameItem)
            self.tableWidget_mat.setItem(row, Const.COL_TYPE, typeItem)
            self.tableWidget_mat.setItem(row, Const.COL_NUMS, numsItem)
            self.tableWidget_mat.setItem(row, Const.COL_SIZE, sizeItem)

    def updateTextFromDataList(self):
        if len(self.dataList) == 0:
            self.resText = ''
            return
        self.resText = '// load from mat file: ' + self.filename + '\n\n'
        for i, data in enumerate(self.dataList):
            if data.shape is ():
                self.resText += stToStr(data)
            else:
                self.resText += arToStr(data)
            if i != len(self.dataList)-1:
                self.resText += '\n'

    def colMoveUp(self):
        row = self.tableWidget_mat.currentRow()
        column = self.tableWidget_mat.currentColumn()
        if row > 0:
            tmp1 = self.dataList[row]
            tmp2 = self.nameList[row]
            self.dataList[row]   = self.dataList[row-1]
            self.nameList[row]   = self.nameList[row-1]
            self.dataList[row-1] = tmp1
            self.nameList[row-1] = tmp2
            self.updateTableFromDataList()
            self.tableWidget_mat.setCurrentCell(row-1,column)

    def colMoveDown(self):
        row = self.tableWidget_mat.currentRow()
        column = self.tableWidget_mat.currentColumn()
        if row < self.tableWidget_mat.rowCount()-1:
            tmp1 = self.dataList[row]
            tmp2 = self.nameList[row]
            self.dataList[row]   = self.dataList[row+1]
            self.nameList[row]   = self.nameList[row+1]
            self.dataList[row+1] = tmp1
            self.nameList[row+1] = tmp2
            self.updateTableFromDataList()
            self.tableWidget_mat.setCurrentCell(row+1,column)

    def colDelete(self):
        if len(self.dataList) == 0:
            return
        row = self.tableWidget_mat.currentRow()
        self.dataList.pop(row)
        self.nameList.pop(row)
        self.updateTableFromDataList()

    def applyNewSeq(self):
        cntRows = self.tableWidget_mat.rowCount()
        # check seq
        for row in range(cntRows):
            try:
                d = int(self.tableWidget_mat.item(row, Const.COL_NEWSQE).text())
            except ValueError:
                return
            if d < 1 or d > cntRows:
                return

        newDataList = list(self.dataList)
        newNameList = list(self.nameList)
        for row in range(cntRows):
            newIdx = int(self.tableWidget_mat.item(row, Const.COL_NEWSQE).text())-1
            newDataList[newIdx] = self.dataList[row]
            newNameList[newIdx] = self.nameList[row]
        self.dataList = newDataList
        self.nameList = newNameList
        self.updateTableFromDataList()

def reStructureData(data):
    f = data.dtype.names
    args = list() # args for new dtype
    dataList = list()
    for i in range(len(f)):
        d = data[f[i]][0][0][0]
        dataList.append(d)
        type = d.dtype.base
        num  = d.size
        args.append(('f'+str(i), type, (num,)))
    return np.array(tuple(dataList), args)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/widget_asa_loader.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WidgetAsaLoader(object):
    def setupUi(self, WidgetAsaLoader):
        WidgetAsaLoader.setObjectName("WidgetAsaLoader")
        WidgetAsaLoader.resize(500, 700)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        WidgetAsaLoader.setFont(font)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(WidgetAsaLoader)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_serial = QtWidgets.QGroupBox(WidgetAsaLoader)
        self.groupBox_serial.setObjectName("groupBox_serial")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_serial)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_selecSerialPort = QtWidgets.QLabel(self.groupBox_serial)
        self.label_selecSerialPort.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label_selecSerialPort.setBaseSize(QtCore.QSize(0, 0))
        self.label_selecSerialPort.setObjectName("label_selecSerialPort")
        self.horizontalLayout_2.addWidget(self.label_selecSerialPort)
        self.comboBox_selectPort = QtWidgets.QComboBox(self.groupBox_serial)
        self.comboBox_selectPort.setObjectName("comboBox_selectPort")
        self.horizontalLayout_2.addWidget(self.comboBox_selectPort)
        self.pushButton_updatePortList = QtWidgets.QPushButton(self.groupBox_serial)
        self.pushButton_updatePortList.setObjectName("pushButton_updatePortList")
        self.horizontalLayout_2.addWidget(self.pushButton_updatePortList)
        self.verticalLayout_3.addWidget(self.groupBox_serial)
        self.groupBox_basicFuns = QtWidgets.QGroupBox(WidgetAsaLoader)
        self.groupBox_basicFuns.setObjectName("groupBox_basicFuns")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_basicFuns)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton_selectFile = QtWidgets.QPushButton(self.groupBox_basicFuns)
        self.pushButton_selectFile.setObjectName("pushButton_selectFile")
        self.gridLayout.addWidget(self.pushButton_selectFile, 0, 1, 1, 1)
        self.lineEdit_selectFile = QtWidgets.QLineEdit(self.groupBox_basicFuns)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.lineEdit_selectFile.setFont(font)
        self.lineEdit_selectFile.setObjectName("lineEdit_selectFile")
        self.gridLayout.addWidget(self.lineEdit_selectFile, 0, 0, 1, 1)
        self.label_steps = QtWidgets.QLabel(self.groupBox_basicFuns)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        self.label_steps.setFont(font)
        self.label_steps.setObjectName("label_steps")
        self.gridLayout.addWidget(self.label_steps, 2, 0, 1, 3)
        self.pushButton_startProg = QtWidgets.QPushButton(self.groupBox_basicFuns)
        self.pushButton_startProg.setObjectName("pushButton_startProg")
        self.gridLayout.addWidget(self.pushButton_startProg, 1, 0, 1, 1)
        self.pushButton_stopProg = QtWidgets.QPushButton(self.groupBox_basicFuns)
        self.pushButton_stopProg.setObjectName("pushButton_stopProg")
        self.gridLayout.addWidget(self.pushButton_stopProg, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.groupBox_basicFuns)
        self.groupBox_specialTools = QtWidgets.QGroupBox(WidgetAsaLoader)
        self.groupBox_specialTools.setObjectName("groupBox_specialTools")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_specialTools)
        self.verticalLayout.setObjectName("verticalLayout")
        self.pushButton_progStk500 = QtWidgets.QPushButton(self.groupBox_specialTools)
        self.pushButton_progStk500.setObjectName("pushButton_progStk500")
        self.verticalLayout.addWidget(self.pushButton_progStk500)
        self.verticalLayout_3.addWidget(self.groupBox_specialTools)
        self.groupBox = QtWidgets.QGroupBox(WidgetAsaLoader)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.progressBar = QtWidgets.QProgressBar(self.groupBox)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_2.addWidget(self.progressBar)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_statusTitle = QtWidgets.QLabel(self.groupBox)
        self.label_statusTitle.setObjectName("label_statusTitle")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_statusTitle)
        self.label_programSizeTitle = QtWidgets.QLabel(self.groupBox)
        self.label_programSizeTitle.setObjectName("label_programSizeTitle")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_programSizeTitle)
        self.label_programSizeContent = QtWidgets.QLabel(self.groupBox)
        self.label_programSizeContent.setObjectName("label_programSizeContent")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.label_programSizeContent)
        self.label_etcTitle = QtWidgets.QLabel(self.groupBox)
        self.label_etcTitle.setObjectName("label_etcTitle")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_etcTitle)
        self.label_etcContent = QtWidgets.QLabel(self.groupBox)
        self.label_etcContent.setObjectName("label_etcContent")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.label_etcContent)
        self.label_statusContent = QtWidgets.QLabel(self.groupBox)
        self.label_statusContent.setObjectName("label_statusContent")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.label_statusContent)
        self.verticalLayout_2.addLayout(self.formLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout_3.addWidget(self.groupBox)

        self.retranslateUi(WidgetAsaLoader)
        QtCore.QMetaObject.connectSlotsByName(WidgetAsaLoader)

    def retranslateUi(self, WidgetAsaLoader):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox_serial.setTitle(_translate("WidgetAsaLoader", "串列埠設定"))
        self.label_selecSerialPort.setText(_translate("WidgetAsaLoader", "選擇串列埠："))
        self.pushButton_updatePortList.setText(_translate("WidgetAsaLoader", "更新串列埠"))
        self.groupBox_basicFuns.setTitle(_translate("WidgetAsaLoader", "燒錄一般應用程式"))
        self.pushButton_selectFile.setText(_translate("WidgetAsaLoader", "選擇檔案"))
        self.label_steps.setText(_translate("WidgetAsaLoader", "<html><head/><body><p>1. 用USB線將ASA_M128與電腦接上。</p><p>2. 將ASA_M128指撥開關撥到&quot;prog&quot;，並按下reset按鈕。</p><p>3. 選擇ASA_M128連接到之串列埠。</p><p>4. 點選選擇檔案，並選取欲燒錄之檔案。</p><p>5. 點選開始燒錄。</p><p>6. 等待下方進度條跑到100%即完成燒錄。</p></body></html>"))
        self.pushButton_startProg.setText(_translate("WidgetAsaLoader", "開始燒錄"))
        self.pushButton_stopProg.setText(_translate("WidgetAsaLoader", "強制終止"))
        self.groupBox_specialTools.setTitle(_translate("WidgetAsaLoader", "燒錄特殊功能應用程式"))
        self.pushButton_progStk500.setText(_translate("WidgetAsaLoader", "燒錄 \"M128_STK500\"  (可將ASA_M128轉變為STK500裝置使用)"))
        self.groupBox.setTitle(_translate("WidgetAsaLoader", "燒錄狀態"))
        self.label_statusTitle.setText(_translate("WidgetAsaLoader", "當前狀態："))
        self.label_programSizeTitle.setText(_translate("WidgetAsaLoader", "程式大小："))
        self.label_programSizeContent.setText(_translate("WidgetAsaLoader", "-"))
        self.label_etcTitle.setText(_translate("WidgetAsaLoader", "預估花費時間："))
        self.label_etcContent.setText(_translate("WidgetAsaLoader", "-"))
        self.label_statusContent.setText(_translate("WidgetAsaLoader", "等待燒錄"))




import numpy as np
from ..hmipac.type import *

# text decode
def getFirstArray(text):
    # TODO: Exception handle?
    lines = text.split('\n')
    status = int(0)
    usedLines = 0
    for i in range(len(lines)):
        s = lines[i]
        # status 0 get type
        if status == 0:
            usedLines += 1
            if isCommentLine(s) or isSpaceLine(s) or isNullLine(s):
                pass
            else:
                typeNum = decodeTypeLine(s)
                status = 1
                resdata = np.array([], getNpType(typeNum))
        # status 1 get data
        elif status == 1:
            usedLines += 1
            if isCommentLine(s) or isSpaceLine(s) or isNullLine(s):
                pass
            else:
                isContinued, data = decodeDataLine(s, typeNum)
                resdata = np.append(resdata, data)
                if isContinued is False:
                    status = 2
        # status 2 remove space lines and null line
        elif status == 2:
            if isSpaceLine(s) or isNullLine(s):
                usedLines += 1
                pass
            else:
                break
    if status == 1:
        raise ValueError()
    return usedLines, resdata

def getFirstStruct(text):
    # TODO: Exception handle?
    lines = text.split('\n')
    usedLines = 0
    status = int(0)
    for i in range(len(lines)):
        usedLines += 1
        # get formatString
        s = lines[i]
        if isCommentLine(s) or isSpaceLine(s) or isNullLine(s):
            pass
        else:
            dt = decodeFsLine(s)
            status  = 1
            break

    # get each type data
    dataList = list()
    for idx in range(len(dt)):
        l, data = getFirstArray('\n'.join(lines[usedLines::]))
        dataList.append(data)
        usedLines += l
        if data.dtype.base.name != dt[idx].base.name:
            raise TypeError()
    resdata = np.array(tuple(dataList), dtype=dt)

    for i in range(len(lines[usedLines+1::])):
        if isSpaceLine(s) or isNullLine(s):
            usedLines += 1
        else:
            break
    return usedLines, resdata

def textToData(text):
    # TODO: Exception handle?
    res = list()
    isContinued = True
    while(isContinued):
        isContinued = False
        try:
            usedLines, data = getFirstArray(text)
        except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
            # print(e)
            pass
        else:
            res.append(data)
            isContinued = True
            text = '\n'.join((text.split('\n'))[usedLines::])
        try:
            usedLines, data = getFirstStruct(text)
        except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
            # print(e)
            pass
        else:
            res.append(data)
            isContinued = True
            text = '\n'.join((text.split('\n'))[usedLines::])
    return res

# line decode ------------------------------------------------------------------
def decodeDataLine(s, typeNum):
    s = removeComment(s)
    s = removeSpace(s)
    isContinued = isLineLastDot(s)
    s = s[:len(s)-isContinued:].split(',')
    return isContinued, np.array(s, getNpType(typeNum))

def decodeTypeLine(s):
    s = removeComment(s)
    s = s.split(':')
    if len(s) != 2:
        return None
    return getTypeNum(removeSpace(s[0]))

def decodeFsLine(s):
    s = removeComment(s)
    s = s.split(':')
    if len(s) != 2:
        return None
    return getStDtype(removeSpace(s[0]))

# is ---------------------------------------------------------------------------
def isNullLine(s):
    return s == ''

def isSpaceLine(s):
    return s.isspace()

def isCommentLine(s):
    s = removeComment(s)
    return s.isspace() or isNullLine(s)

def isLineLastDot(s):
    if s[-1] == ',':
        return True
    return False

def isSpace(ch):
    return ch.isspace()

# remove -----------------------------------------------------------------------
def removeSpace(s):
    start = int()
    end   = int()
    for i in range(len(s)):
        if isSpace(s[i]):
            pass
        else:
            start = i
            break
    for i in range(len(s))[::-1]:
        if isSpace(s[i]):
            pass
        else:
            end = i+1
            break
    return s[start:end]

def removeComment(s):
    end = len(s)
    for i in range(len(s)):
        if s[i]=='/' and s[i+1]=='/':
            end = i
            break
    return s[:end:]



import enum

class AdtCmd(enum.IntEnum):
    TERM = 1
    LOADER = 2
    STK500 = 3

class AdtSubCmdLoader(enum.IntEnum):
    START = 1
    STATE = 2

class AdtSubCmdTerm(enum.IntEnum):
    OPEN  = 1
    CLOSE = 2
    CLEAR = 3



import scipy.io
from PyQt5 import QtCore
from PyQt5.QtWidgets import QFileDialog, QDialog, QTableWidgetItem
from asa_hmi_data_agent.ui.ui_hmi_save_dialog import Ui_HmiSaveDialog
# import asa_hmi_data_agent.hmi.decodeASAformat as ds
# import asa_hmi_data_agent.hmi.text_decoder as hmidecoder
# import ..hmipac

import numpy as np
from numpy import array
from .text_to_data import textToData
from ..hmipac.type import *

# Reference : https://docs.scipy.org/doc/scipy/reference/tutorial/io.html
# Reference : https://docs.scipy.org/doc/numpy-1.13.0/user/basics.types.html

class Const:
    COL_NAME = 0
    COL_TYPE = 1
    COL_NUMS = 2
    COL_SIZE = 3

# ---- class BitsSelector Start ------------------------------------------------
class HmiSaveDialog(QDialog, Ui_HmiSaveDialog):
    def __init__(self):
        QDialog.__init__(self)
        self.setupUi(self)
        self.pushButton_txtClose.clicked.connect(self.accept)
        self.pushButton_txtSave.clicked.connect(self.saveAsTxt)
        self.pushButton_matClose.clicked.connect(self.accept)
        self.pushButton_matSave.clicked.connect(self.saveAsMat)
        self.dataList = list()

    def show(self):
        super(QDialog, self).show()

    def showAndLoadText(self, text):
        self.show()
        self.loadDataFromText(text)

    def loadDataFromText(self, text):
        # tab_txt
        self.textEdit_txt.append(text)

        # tab_mat
        try:
            self.dataList = textToData(text)
        except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
            # TODO ERROR Dialog
            print(type(e))
            self.accept()
            pass
        else:
            self.tableWidget_mat.setRowCount(len(self.dataList))
            for i in range(len(self.dataList)):
                data = self.dataList[i]
                nameStr = 'data' + str(i)
                if data.shape is ():
                    # Struct
                    typeStr = getFs(data.dtype)
                    numsStr = '1'
                    sizeStr = str(data.dtype.itemsize)
                else:
                    # Array
                    typeStr = getTypeStr(getTypeNum(data.dtype.name))
                    numsStr = str(data.size)
                    sizeStr = str(data.size * data.dtype.itemsize)

                nameItem = QTableWidgetItem(nameStr)
                typeItem = QTableWidgetItem(typeStr)
                numsItem = QTableWidgetItem(numsStr)
                sizeItem = QTableWidgetItem(sizeStr)
                # nameItem.setFlags(QtCore.Qt.ItemIsEnabled)
                typeItem.setFlags(QtCore.Qt.ItemIsEnabled)
                numsItem.setFlags(QtCore.Qt.ItemIsEnabled)
                sizeItem.setFlags(QtCore.Qt.ItemIsEnabled)
                self.tableWidget_mat.setItem(i, Const.COL_NAME, nameItem)
                self.tableWidget_mat.setItem(i, Const.COL_TYPE, typeItem)
                self.tableWidget_mat.setItem(i, Const.COL_NUMS, numsItem)
                self.tableWidget_mat.setItem(i, Const.COL_SIZE, sizeItem)

    def saveAsMat(self):
        d = dict()
        for i in range(len(self.dataList)):
            name = self.tableWidget_mat.item(i, Const.COL_NAME).text()
            data = self.dataList[i]
            d[name] = data

        name, _ = QFileDialog.getSaveFileName(self, 'Save File','', 'All Files (*);;Mat Files (*.mat)' ,initialFilter='Mat Files (*.mat)')
        if name is not '':
            scipy.io.savemat(name, d)

    def saveAsTxt(self):
        name, _ = QFileDialog.getSaveFileName(self, 'Save File','', 'All Files (*);;txt Files (*.txt)' ,initialFilter='txt Files (*.txt)')
        if name is not '':
            with open(name, 'w') as f:
                f.write(self.textEdit_txt.toPlainText())



from asa_hmi_data_agent.ui.ui_avrdude import Ui_MainWidgetAvrdude
from asa_hmi_data_agent.ui.ui_bit_selector import Ui_BitSelector
from asa_hmi_data_agent.avrdude.avrdudeConfParser import AvrdudeConfParser
from asa_hmi_data_agent.util import ADTPATH
from asa_hmi_data_agent.listport import getAvailableSerialPorts
from configparser import ConfigParser

from PyQt5.QtCore import QObject, QThread, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QFileDialog, QDialog
from PyQt5.QtGui import QTextCursor

import subprocess
import datetime
import serial
import time

# ---- class ShellThread Start -------------------------------------------------
class ShellThread(QThread):
    shellIsRunning = False

    signalGetLine = pyqtSignal(str)
    signalReadFuseDone = pyqtSignal()
    signalReadLockDone = pyqtSignal()

    def __init__(self):
        QThread.__init__(self)
        self.cmd = str()
        self.cmdType = 0

    def setCmd(self, cmd):
        self.cmd = cmd

    def setCmdType(self, num):
        self.cmdType = num

    def run(self):
        times = time.strftime("%H:%M:%S", time.gmtime())
        self.signalGetLine.emit('[' + times + '] ' + self.cmd + '\n')
        self.shellIsRunning = True

        debugLog(str(self.cmd.split(' ')))
        self.p = subprocess.Popen(self.cmd.split(' ') , stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE, shell=True)
        while self.p.poll() is None:
            s = self.p.stderr.readline()
            if(s.decode("big5") is not ''):
                self.signalGetLine.emit(s.decode("big5"))
        # Send Complete singal if needed
        if self.cmdType is 1:
            self.cmdType = 0
            self.signalReadFuseDone.emit()
        if self.cmdType is 2:
            self.cmdType = 0
            self.signalReadLockDone.emit()
        # Send Complete message
        times = time.strftime("%H:%M:%S", time.gmtime())
        self.signalGetLine.emit('[' + times + '] ' +  'Complete!' + '\n\n')
        self.shellIsRunning = False

    def stop(self):
        try:
            if self.shellIsRunning or self.p.poll() is None:
                times = time.strftime("%H:%M:%S", time.gmtime())
                self.signalGetLine.emit('[' + times + '] ' +  'Terminate program!' + '\n')
                self.shellIsRunning = False
                self.p.kill()
            else:
                pass
        except AttributeError as e:
            pass
        self.terminate()

# ---- class ShellThread End ---------------------------------------------------

# ---- class BitsSelector Start ------------------------------------------------
class BitsSelector(QDialog, Ui_BitSelector):
    def __init__(self):
        QDialog.__init__(self)
        self.setupUi(self)
        self.pushButton_close.clicked.connect(self.accept)
        self.chkFuseL = list()
        self.chkFuseL.append(self.checkBox_fuseL0)
        self.chkFuseL.append(self.checkBox_fuseL1)
        self.chkFuseL.append(self.checkBox_fuseL2)
        self.chkFuseL.append(self.checkBox_fuseL3)
        self.chkFuseL.append(self.checkBox_fuseL4)
        self.chkFuseL.append(self.checkBox_fuseL5)
        self.chkFuseL.append(self.checkBox_fuseL6)
        self.chkFuseL.append(self.checkBox_fuseL7)
        self.chkFuseH = list()
        self.chkFuseH.append(self.checkBox_fuseH0)
        self.chkFuseH.append(self.checkBox_fuseH1)
        self.chkFuseH.append(self.checkBox_fuseH2)
        self.chkFuseH.append(self.checkBox_fuseH3)
        self.chkFuseH.append(self.checkBox_fuseH4)
        self.chkFuseH.append(self.checkBox_fuseH5)
        self.chkFuseH.append(self.checkBox_fuseH6)
        self.chkFuseH.append(self.checkBox_fuseH7)
        self.chkFuseE = list()
        self.chkFuseE.append(self.checkBox_fuseE0)
        self.chkFuseE.append(self.checkBox_fuseE1)
        self.chkFuseE.append(self.checkBox_fuseE2)
        self.chkFuseE.append(self.checkBox_fuseE3)
        self.chkFuseE.append(self.checkBox_fuseE4)
        self.chkFuseE.append(self.checkBox_fuseE5)
        self.chkFuseE.append(self.checkBox_fuseE6)
        self.chkFuseE.append(self.checkBox_fuseE7)
        self.chkLock = list()
        self.chkLock.append(self.checkBox_lock0)
        self.chkLock.append(self.checkBox_lock1)
        self.chkLock.append(self.checkBox_lock2)
        self.chkLock.append(self.checkBox_lock3)
        self.chkLock.append(self.checkBox_lock4)
        self.chkLock.append(self.checkBox_lock5)
        self.chkLock.append(self.checkBox_lock6)
        self.chkLock.append(self.checkBox_lock7)
        self.lineEdit_fuseLow  .textChanged.connect(lambda: self.updateChkFromLine(self.chkFuseL, self.lineEdit_fuseLow))
        self.lineEdit_fuseHigh .textChanged.connect(lambda: self.updateChkFromLine(self.chkFuseH, self.lineEdit_fuseHigh))
        self.lineEdit_fuseExtra.textChanged.connect(lambda: self.updateChkFromLine(self.chkFuseE, self.lineEdit_fuseExtra))
        self.lineEdit_lock     .textChanged.connect(lambda: self.updateChkFromLine(self.chkLock , self.lineEdit_lock))
        for bit in range(0,8):
            self.chkFuseL[bit].clicked.connect(lambda:self.updateLineFromChk(self.lineEdit_fuseLow, self.chkFuseL))
        for bit in range(0,8):
            self.chkFuseH[bit].clicked.connect(lambda:self.updateLineFromChk(self.lineEdit_fuseHigh, self.chkFuseH))
        for bit in range(0,8):
            self.chkFuseE[bit].clicked.connect(lambda:self.updateLineFromChk(self.lineEdit_fuseExtra, self.chkFuseE))
        for bit in range(0,8):
            self.chkLock[bit].clicked.connect(lambda:self.updateLineFromChk(self.lineEdit_lock, self.chkLock))

    def show(self, partDesc):
        super(QDialog, self).show()
        config = ConfigParser()
        config.read(ADTPATH+'/settings/bits_info.ini')
        try:
            bitInfoList = config[partDesc]['fuse_l'].replace(' ', '').split(',')
            for bit in range(0,8):
                self.chkFuseL[bit].setChecked(int(self.lineEdit_fuseLow.text(), 16) & (1<<bit))
                if bitInfoList[bit] is not '':
                    self.chkFuseL[bit].setText(bitInfoList[bit])
                    self.chkFuseL[bit].setStyleSheet("color: rgb(0, 0, 0);")
                else:
                    self.chkFuseL[bit].setText('bit' + str(bit))
                    self.chkFuseL[bit].setStyleSheet("color: rgb(128, 128, 128);")
        except (ValueError, KeyError):
            pass
        try:
            bitInfoList = config[partDesc]['fuse_h'].replace(' ', '').split(',')
            for bit in range(0,8):
                self.chkFuseH[bit].setChecked(int(self.lineEdit_fuseHigh.text(), 16) & (1<<bit))
                if bitInfoList[bit] is not '':
                    self.chkFuseH[bit].setText(bitInfoList[bit])
                    self.chkFuseH[bit].setStyleSheet("color: rgb(0, 0, 0);")
                else:
                    self.chkFuseH[bit].setText('bit' + str(bit))
                    self.chkFuseH[bit].setStyleSheet("color: rgb(128, 128, 128);")
        except (ValueError, KeyError):
            pass
        try:
            bitInfoList = config[partDesc]['fuse_e'].replace(' ', '').split(',')
            for bit in range(0,8):
                self.chkFuseE[bit].setChecked(int(self.lineEdit_fuseExtra.text(), 16) & (1<<bit))
                if bitInfoList[bit] is not '':
                    self.chkFuseE[bit].setText(bitInfoList[bit])
                    self.chkFuseE[bit].setStyleSheet("color: rgb(0, 0, 0);")
                else:
                    self.chkFuseE[bit].setText('bit' + str(bit))
                    self.chkFuseE[bit].setStyleSheet("color: rgb(128, 128, 128);")
        except (ValueError, KeyError):
            pass
        try:
            bitInfoList = config[partDesc]['lock'].replace(' ', '').split(',')
            for bit in range(0,8):
                self.chkLock[bit].setChecked(int(self.lineEdit_lock.text(), 16) & (1<<bit))
                if bitInfoList[bit] is not '':
                    self.chkLock[bit].setText(bitInfoList[bit])
                    self.chkLock[bit].setStyleSheet("color: rgb(0, 0, 0);")
                else:
                    self.chkLock[bit].setText('bit' + str(bit))
                    self.chkLock[bit].setStyleSheet("color: rgb(128, 128, 128);")
        except (ValueError, KeyError):
            pass

    def updateChkFromLine(self, chkList, lineEdit):
        try:
            for bit in range(0,8):
                chkList[bit].setChecked(int(lineEdit.text(), 16) & (1<<bit))
        except ValueError:
            pass

    def updateLineFromChk(self, lineEdit, chkList):
        try:
            val = 0
            for bit in range(0,8):
                if chkList[bit].isChecked() is True:
                    val = val + (1<<bit)
            lineEdit.setText("0x%02X" % val)
        except ValueError:
            pass
# ---- class BitsSelector End --------------------------------------------------

def radioButtonClick(btn):
    if btn.isChecked() is True:
        btn.setAutoExclusive(False)
        btn.setChecked(False)
        btn.setAutoExclusive(True)

# ---- class Avrdude Start -----------------------------------------------------
class Avrdude(QObject):
    sigChangeWindowTitle = pyqtSignal(str)
    sigSerialPortCheck = pyqtSignal(bool, str)

    # ---- __init__ start ------------------------------------------------------
    def __init__(self, widget):
        super(Avrdude, self).__init__()
        self.ui = Ui_MainWidgetAvrdude()
        self.ui.setupUi(widget)

        # ---- Settings Group start --------------------------------------------
        self.settingsFile = ADTPATH+'/settings/avrdude_settings.ini'
        self.conf = ConfigParser()
        self.settingsListUpdate()
        self.settingsLoad()
        self.ui.pushButton_configSave.clicked.connect(self.settingsSave)
        self.ui.pushButton_configDelete.clicked.connect(self.settingsDelete)
        self.ui.comboBox_config.currentIndexChanged.connect(self.settingsLoad)
        # ---- Settings Group end ----------------------------------------------

        # ---- Serial object Init Start ----------------------------------------
        self.ser = serial.Serial()
        self.ser.baudrate = 38400
        self.ser.timeout = 10
        # ---- Serial object Init End ------------------------------------------

        # ---- Shell Thread Init Start -----------------------------------------
        self.shellThread = ShellThread()
        self.shellThread.signalGetLine[str].connect(self.terminalAppendLine)
        # self.shellThread.signalGetLine[str].connect(self.terminalAppendLine)
        self.shellThread.started.connect(
            lambda : self.sigChangeWindowTitle.emit(True, self.ui.comboBox_serialSetPort.currentText())
        )
        self.shellThread.finished.connect(
            lambda: self.sigChangeWindowTitle.emit(
                False, self.ui.comboBox_serialSetPort.currentText())
        )
        # ---- Shell Thread Init End -------------------------------------------

        # ---- Serial Group start ----------------------------------------------
        self.serial_updatePortlist()
        self.ui.pushButton_updatePort.clicked.connect(self.serial_updatePortlist)
        self.ui.lineEdit_serialSetBaud.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_serialSetBaud.setText('38400')
        # ---- Serial Group end ------------------------------------------------

        # ---- Flash Group start -----------------------------------------------
        self.ui.toolButton_flash.clicked.connect(self.flash_chooseBinaryFile)
        self.ui.pushButton_flashGo.clicked.connect(self.flash_execute)
        self.ui.radioButton_flashWrite.clicked.connect (lambda:self.flash_radioButtonClick(self.ui.radioButton_flashWrite))
        self.ui.radioButton_flashRead.clicked.connect  (lambda:self.flash_radioButtonClick(self.ui.radioButton_flashRead))
        self.ui.radioButton_flashVerify.clicked.connect(lambda:self.flash_radioButtonClick(self.ui.radioButton_flashVerify))
        # ---- Flash Group end -------------------------------------------------

        # ---- Eeprom Group start ----------------------------------------------
        self.ui.toolButton_eeprom.clicked.connect(self.eeprom_chooseBinaryFile)
        self.ui.pushButton_eepromGo.clicked.connect(self.eeprom_execute)
        self.ui.radioButton_eepromWrite.clicked.connect (lambda:self.eeprom_radioButtonClick(self.ui.radioButton_eepromWrite))
        self.ui.radioButton_eepromRead.clicked.connect  (lambda:self.eeprom_radioButtonClick(self.ui.radioButton_eepromRead))
        self.ui.radioButton_eepromVerify.clicked.connect(lambda:self.eeprom_radioButtonClick(self.ui.radioButton_eepromVerify))
        # ---- Eeprom Group end ------------------------------------------------

        # ---- Fuse & Lock Group start ---------------------------------------
        self.shellThread.signalReadFuseDone.connect(self.fuse_update_from_tmp)
        self.shellThread.signalReadLockDone.connect(self.lock_update_from_tmp)
        self.ui.lineEdit_lock.setText('0x00')
        self.ui.lineEdit_fuseHigh.setText('0x00')
        self.ui.lineEdit_fuseLow.setText('0x00')
        self.ui.lineEdit_fuseExtra.setText('0x00')
        self.ui.pushButton_fuseRead.clicked.connect(self.fuse_read)
        self.ui.pushButton_fuseWrite.clicked.connect(self.fuse_write)
        self.ui.pushButton_lockRead.clicked.connect(self.lock_read)
        self.ui.pushButton_flashGo.clicked.connect(self.lock_write)
        self.ui.pushButton_bitSelector.clicked.connect(self.bitsSelectorShow)
        # ---- Fuse & Lock Group end -----------------------------------------

        # ---- MCU Group start -------------------------------------------------
        self.devices = AvrdudeConfParser().GetDeviceInfo()

        self.ui.pushButton_mcuDetect.clicked.connect(self.mcu_detect)
        self.ui.comboBox_mcuSelect.clear()
        self.ui.comboBox_mcuSelect.addItem('請選擇MCU...')
        for device in self.devices:
            self.ui.comboBox_mcuSelect.addItem(device['desc'])
        # ---- MCU Group end ---------------------------------------------------

        # ---- need updateCammand items start ----------------------------------
        self.ui.comboBox_serialSetPort.currentIndexChanged.connect(self.updateCammand)
        self.ui.comboBox_mcuSelect.currentIndexChanged.connect(self.updateCammand)

        self.ui.checkBox_lockSet.clicked.connect(self.updateCammand)
        self.ui.checkBox_fuseSet.clicked.connect(self.updateCammand)
        self.ui.checkBox_eraseChip.clicked.connect(self.updateCammand)
        self.ui.checkBox_cancelVerify.clicked.connect(self.updateCammand)

        self.ui.lineEdit_lock.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_flash.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_eeprom.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_fuseHigh.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_fuseLow.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_fuseExtra.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_serialSetBaud.textChanged.connect(self.updateCammand)
        self.ui.lineEdit_additionalParameter.textChanged.connect(self.updateCammand)
        # ---- need updateCammand items end ------------------------------------

        self.ui.checkBox_eraseChip.setChecked(True)
        self.updateCammand()
        self.ui.pushButton_startProgram.clicked.connect(self.startProgram)
        self.ui.pushButton_stopProgram.clicked.connect(self.stopProgram)

        # ---- BitsSelector start ----------------------------------------------
        self.bitsSelector = BitsSelector()
        self.bitsSelector.accepted.connect(self.updateBitsFromSelector)
        # ---- BitsSelector end ------------------------------------------------

    # ---- Methods Section start -----------------------------------------------
    def startProgram(self):
        self.updateCammand()
        self.shellThread.setCmd(self.ui.textBrowser_cmd.toPlainText())
        self.shellThread.start()

    def stopProgram(self):
        self.shellThread.stop()

    def getBasicParameter(self):
        cmd = str()
        cmd += ADTPATH+'/tools/avrdude'
        cmd += ' -c stk500'

        if  self.ui.comboBox_mcuSelect.currentIndex() > 0:
            desc = self.ui.comboBox_mcuSelect.currentText()
            id = [device for device in self.devices if device['desc'] == desc][0]['id']
            cmd += ' -p ' + id

        cmd += ' -b ' + self.ui.lineEdit_serialSetBaud.text()

        if self.ui.comboBox_serialSetPort.currentText() is not '':
            cmd += ' -P ' + self.ui.comboBox_serialSetPort.currentText()
        return cmd

    # check all items and update cmd in line
    def updateCammand(self):
        cmd = str()
        cmd += 'tools\\avrdude'
        cmd += ' -c stk500'

        if  self.ui.comboBox_mcuSelect.currentIndex() > 0:
            desc = self.ui.comboBox_mcuSelect.currentText()
            id = [device for device in self.devices if device['desc'] == desc][0]['id']
            cmd += ' -p ' + id

        cmd += ' -b ' + self.ui.lineEdit_serialSetBaud.text()

        if self.ui.comboBox_serialSetPort.currentText() is not '':
            cmd += ' -P ' + self.ui.comboBox_serialSetPort.currentText()

        if self.ui.checkBox_eraseChip.isChecked():
            cmd += ' -e'

        if self.ui.checkBox_cancelVerify.isChecked():
            cmd += ' -V'

        if self.ui.lineEdit_additionalParameter.text() is not '':
            cmd += ' ' + self.ui.lineEdit_additionalParameter.text()

        tmp = self.flash_radioButtonCheck()
        if tmp is not '':
            cmd += ' -U flash:' + tmp + ':' + self.ui.lineEdit_flash.text() + ':i'

        tmp = self.eeprom_radioButtonCheck()
        if tmp is not '':
            cmd += ' -U eeprom:' + tmp + ':' + self.ui.lineEdit_flash.text() + ':i'

        if self.ui.checkBox_fuseSet.isChecked():
            cmd += ' -U hfuse:w:' + self.ui.lineEdit_fuseHigh.text()  + ':m'
            cmd += ' -U lfuse:w:' + self.ui.lineEdit_fuseLow.text()   + ':m'
            cmd += ' -U efuse:w:' + self.ui.lineEdit_fuseExtra.text() + ':m'

        if self.ui.checkBox_lockSet.isChecked():
            cmd += ' -U lock:w:' + self.ui.lineEdit_lock.text() + ':m'

        self.ui.textBrowser_cmd.clear()
        self.ui.textBrowser_cmd.append(cmd)

    def terminalAppendLine(self, s):
        text = self.ui.textBrowser_cmdterminal.toPlainText() + s
        self.ui.textBrowser_cmdterminal.clear()
        self.ui.textBrowser_cmdterminal.setText(text)
        self.ui.textBrowser_cmdterminal.moveCursor(QTextCursor.End)
    # ---- Settings Group start ------------------------------------------------
    def settingsListUpdate(self):
        self.conf.read(self.settingsFile)
        self.ui.comboBox_config.clear()
        for section in self.conf.sections():
            self.ui.comboBox_config.addItem(section)

    def settingsSave(self):
        section = self.ui.comboBox_config.currentText()
        if self.conf.has_section(section) is False:
            self.conf.add_section(section)
        self.conf.set(section, 'port', self.ui.comboBox_serialSetPort.currentText())
        self.conf.set(section, 'baud', self.ui.lineEdit_serialSetBaud.text())
        self.conf.set(section, 'mcu', self.ui.comboBox_mcuSelect.currentText())
        self.conf.set(section, 'flashFile', self.ui.lineEdit_flash.text())
        self.conf.set(section, 'flashExecType', self.flash_radioButtonCheck())
        self.conf.set(section, 'eepromFile', self.ui.lineEdit_eeprom.text())
        self.conf.set(section, 'eepromExecType', self.eeprom_radioButtonCheck())
        self.conf.set(section, 'fuseL', self.ui.lineEdit_fuseLow.text())
        self.conf.set(section, 'fuseH', self.ui.lineEdit_fuseHigh.text())
        self.conf.set(section, 'fuseE', self.ui.lineEdit_fuseExtra.text())
        self.conf.set(section, 'lock', self.ui.lineEdit_lock.text())
        self.conf.set(section, 'fuseSet', str(self.ui.checkBox_fuseSet.isChecked()))
        self.conf.set(section, 'lockSet', str(self.ui.checkBox_lockSet.isChecked()))

        self.conf.set(section, 'cancelVerify', str(self.ui.checkBox_cancelVerify.isChecked()))
        self.conf.set(section, 'earseChip',    str(self.ui.checkBox_eraseChip.isChecked()))
        self.conf.set(section, 'additionalParameter', self.ui.lineEdit_additionalParameter.text())

        with open(self.settingsFile, 'w') as configfile:
            self.conf.write(configfile)
        self.settingsListUpdate()
        self.ui.comboBox_config.setCurrentText(section)
        self.settingsLoad()

    def settingsLoad(self):
        section = self.ui.comboBox_config.currentText()
        if section is '':
            return
        self.ui.comboBox_serialSetPort.setCurrentText(self.conf[section]['port'])
        self.ui.lineEdit_serialSetBaud.setText(self.conf[section]['baud'])
        self.ui.comboBox_mcuSelect.setCurrentText(self.conf[section]['mcu'])
        self.ui.lineEdit_flash.setText(self.conf[section]['flashfile'])
        self.flash_radioButtonSet(self.conf[section]['flashexectype'])
        self.ui.lineEdit_eeprom.setText(self.conf[section]['eepromfile'])
        self.eeprom_radioButtonSet(self.conf[section]['eepromexectype'])
        self.ui.lineEdit_fuseLow.setText(self.conf[section]['fusel'])
        self.ui.lineEdit_fuseHigh.setText(self.conf[section]['fuseh'])
        self.ui.lineEdit_fuseExtra.setText(self.conf[section]['fusee'])
        self.ui.lineEdit_lock.setText(self.conf[section]['lock'])
        self.ui.checkBox_fuseSet.setChecked(self.conf.getboolean(section,'fuseset'))
        self.ui.checkBox_lockSet.setChecked(self.conf.getboolean(section,'lockset'))
        self.ui.checkBox_cancelVerify.setChecked(self.conf.getboolean(section,'cancelverify'))
        self.ui.checkBox_eraseChip.setChecked(self.conf.getboolean(section,'earsechip'))
        self.ui.lineEdit_additionalParameter.setText(self.conf[section]['additionalparameter'])

    def settingsDelete(self):
        section = self.ui.comboBox_config.currentText()
        self.conf.remove_section(section)
        with open(self.settingsFile, 'w') as configfile:
            self.conf.write(configfile)
        self.settingsListUpdate()
    # ---- Settings Group end --------------------------------------------------

    # ---- Serial Group start --------------------------------------------------
    # Update port list in s_portComboBox
    def serial_updatePortlist(self):
        availablePorts = getAvailableSerialPorts()
        debugLog('Update ports: ' + str(availablePorts))
        self.ui.comboBox_serialSetPort.clear()
        for port in availablePorts:
            self.ui.comboBox_serialSetPort.addItem(port)
    # ---- Serial Group end ----------------------------------------------------

    # ---- Flash Group start ---------------------------------------------------
    def flash_chooseBinaryFile(self):
        name, _ = QFileDialog.getOpenFileName(
            filter='All Files (*);;Hex Files (*.hex)',
            initialFilter='Hex Files (*.hex)'
        )
        if name is not '':
            self.ui.lineEdit_flash.setText(name)

    def flash_radioButtonClick(self,btn):
        if btn is not self.ui.radioButton_flashWrite:
            self.ui.radioButton_flashWrite.setChecked(0)
        if btn is not self.ui.radioButton_flashRead:
            self.ui.radioButton_flashRead.setChecked(0)
        if btn is not self.ui.radioButton_flashVerify:
            self.ui.radioButton_flashVerify.setChecked(0)
        self.updateCammand();

    def flash_radioButtonCheck(self):
        if self.ui.radioButton_flashWrite.isChecked():
            return 'w'
        elif self.ui.radioButton_flashRead.isChecked():
            return 'r'
        elif self.ui.radioButton_flashVerify.isChecked():
            return 'v'
        else:
            return ''

    def flash_radioButtonSet(self, execType):
        if execType is 'w':
            self.ui.radioButton_flashWrite.setChecked(True)
            self.ui.radioButton_flashRead.setChecked(False)
            self.ui.radioButton_flashVerify.setChecked(False)
        elif execType is 'r':
            self.ui.radioButton_flashWrite.setChecked(False)
            self.ui.radioButton_flashRead.setChecked(True)
            self.ui.radioButton_flashVerify.setChecked(False)
        elif execType is 'v':
            self.ui.radioButton_flashWrite.setChecked(False)
            self.ui.radioButton_flashRead.setChecked(False)
            self.ui.radioButton_flashVerify.setChecked(True)
        else:
            self.ui.radioButton_flashWrite.setChecked(False)
            self.ui.radioButton_flashRead.setChecked(False)
            self.ui.radioButton_flashVerify.setChecked(False)

    def flash_execute(self):
        cmd = self.getBasicParameter()
        tmp = self.flash_radioButtonCheck()
        if tmp is not '':
            cmd += ' -U flash:' + tmp + ':' + self.ui.lineEdit_flash.text() + ':i'
        times = time.strftime("%H:%M:%S", time.gmtime())
        self.terminalAppendLine('[' + times + '] ' + 'Flash Go!' + '\n')
        self.shellThread.setCmd(cmd)
        self.shellThread.start()
    # ---- Flash Group end -----------------------------------------------------

    # ---- Eeprom Group start --------------------------------------------------
    def eeprom_chooseBinaryFile(self):
        name, _ = QFileDialog.getOpenFileName(
            filter='All Files (*);;Hex Files (*.hex)',
            initialFilter='Hex Files (*.hex)'
        )
        if name is not '':
            self.ui.lineEdit_eeprom.setText(name)

    def eeprom_radioButtonClick(self,btn):
        if btn is not self.ui.radioButton_eepromWrite:
            self.ui.radioButton_eepromWrite.setChecked(0)
        if btn is not self.ui.radioButton_eepromRead:
            self.ui.radioButton_eepromRead.setChecked(0)
        if btn is not self.ui.radioButton_eepromVerify:
            self.ui.radioButton_eepromVerify.setChecked(0)
        self.updateCammand();

    def eeprom_radioButtonCheck(self):
        if self.ui.radioButton_eepromWrite.isChecked():
            return 'w'
        elif self.ui.radioButton_eepromRead.isChecked():
            return 'r'
        elif self.ui.radioButton_eepromVerify.isChecked():
            return 'v'
        else:
            return ''

    def eeprom_radioButtonSet(self, execType):
        if execType is 'w':
            self.ui.radioButton_eepromWrite.setChecked(True)
            self.ui.radioButton_eepromRead.setChecked(False)
            self.ui.radioButton_eepromVerify.setChecked(False)
        elif execType is 'r':
            self.ui.radioButton_eepromWrite.setChecked(False)
            self.ui.radioButton_eepromRead.setChecked(True)
            self.ui.radioButton_eepromVerify.setChecked(False)
        elif execType is 'v':
            self.ui.radioButton_eepromWrite.setChecked(False)
            self.ui.radioButton_eepromRead.setChecked(False)
            self.ui.radioButton_eepromVerify.setChecked(True)
        else:
            self.ui.radioButton_eepromWrite.setChecked(False)
            self.ui.radioButton_eepromRead.setChecked(False)
            self.ui.radioButton_eepromVerify.setChecked(False)

    def eeprom_execute(self):
        cmd = self.getBasicParameter()
        tmp = self.eeprom_radioButtonCheck()
        if tmp is not '':
            cmd += ' -U eeprom:' + tmp + ':' + self.ui.lineEdit_flash.text() + ':i'
        times = time.strftime("%H:%M:%S", time.gmtime())
        self.terminalAppendLine('[' + times + '] ' + 'EEPROM Go!' + '\n')
        self.shellThread.setCmd(cmd)
        self.shellThread.start()
    # ---- Eeprom Group end ----------------------------------------------------

    # ---- MCU Group start -----------------------------------------------------
    # detect MCU is correct or wrong
    def mcu_detect(self):
        cmd = self.getBasicParameter()
        self.shellThread.setCmd(cmd)
        self.shellThread.start()
    # ---- MCU Group end -------------------------------------------------------

    # ---- Fuse & Lock Group start ---------------------------------------------
    def fuse_read(self):
        cmd = self.getBasicParameter()
        cmd += ' -U lfuse:r:' + ADTPATH + '/tmp/fuse_low.txt'   + ':h'
        cmd += ' -U hfuse:r:' + ADTPATH + '/tmp/fuse_high.txt'  + ':h'
        cmd += ' -U efuse:r:' + ADTPATH + '/tmp/fuse_extra.txt' + ':h'
        self.shellThread.setCmdType(1)
        self.shellThread.setCmd(cmd)
        self.shellThread.start()

    def fuse_update_from_tmp(self):
        fl = open(ADTPATH+'/tmp/fuse_low.txt', 'r')
        fh = open(ADTPATH+'/tmp/fuse_high.txt', 'r')
        fe = open(ADTPATH+'/tmp/fuse_extra.txt', 'r')
        # self.ui.lineEdit_lock.setText()
        self.ui.lineEdit_fuseLow.setText(fl.read(4))
        self.ui.lineEdit_fuseHigh.setText(fh.read(4))
        self.ui.lineEdit_fuseExtra.setText(fe.read(4))
        fl.close()
        fh.close()
        fe.close()

    def fuse_write(self):
        cmd = self.getBasicParameter()
        cmd += ' -U lfuse:w:' + self.ui.lineEdit_fuseLow.text()   + ':m'
        cmd += ' -U hfuse:w:' + self.ui.lineEdit_fuseHigh.text()  + ':m'
        cmd += ' -U efuse:w:' + self.ui.lineEdit_fuseExtra.text() + ':m'
        self.shellThread.setCmd(cmd)
        self.shellThread.start()

    def lock_read(self):
        cmd = self.getBasicParameter()
        cmd += ' -U lock:r:' + ADTPATH + '/tmp/lock.txt' + ':h'
        self.shellThread.setCmd(cmd)
        self.shellThread.start()

    def lock_update_from_tmp(self):
        f = open(ADTPATH + '/tmp/lock.txt', 'r')
        self.ui.lineEdit_lock.setText(f.read(4))
        f.close()

    def lock_write(self):
        cmd = self.getBasicParameter()
        cmd += ' -U hfuse:w:' + self.ui.lineEdit_lock.text()  + ':m'
        self.shellThread.setCmdType(2)
        self.shellThread.setCmd(cmd)
        self.shellThread.start()
    # ---- Fuse & Lock Group end -----------------------------------------------

    # ---- BitsSelector start --------------------------------------------------
    def updateBitsFromSelector(self):
        self.ui.lineEdit_fuseHigh .setText(self.bitsSelector.lineEdit_fuseHigh .text())
        self.ui.lineEdit_fuseLow  .setText(self.bitsSelector.lineEdit_fuseLow  .text())
        self.ui.lineEdit_fuseExtra.setText(self.bitsSelector.lineEdit_fuseExtra.text())
        self.ui.lineEdit_lock     .setText(self.bitsSelector.lineEdit_lock.text())

        pass
    def bitsSelectorShow(self):
        self.bitsSelector.lineEdit_fuseHigh .setText(self.ui.lineEdit_fuseHigh .text())
        self.bitsSelector.lineEdit_fuseLow  .setText(self.ui.lineEdit_fuseLow  .text())
        self.bitsSelector.lineEdit_fuseExtra.setText(self.ui.lineEdit_fuseExtra.text())
        self.bitsSelector.lineEdit_lock.setText(self.ui.lineEdit_lock.text())
        self.bitsSelector.show(self.ui.comboBox_mcuSelect.currentText())
    # ---- BitsSelector end ----------------------------------------------------
    # ---- Methods Section end -------------------------------------------------
# ---- class Avrdude End -------------------------------------------------------

def debugLog(msg):
    s = '[{}] avrdude  log: {}'.format(
        datetime.datetime.now().strftime("%H:%M:%S"),
        msg
    )
    print(s)



from asa_hmi_data_agent.cli_tools.commands import *
from asa_hmi_data_agent.cli_tools.socket_handler import SocketHandler

import argparse

def argHandler():
    parser = argparse.ArgumentParser(description='Controll the terminal in adt.')
    subparsers = parser.add_subparsers(dest='subcmd')

    # parser for the "open" command
    parser_open = subparsers.add_parser('open', help='Open the terminal in adt.')
    parser_open.add_argument(
        '-P', '-p', '--port',
        dest='port', action ='store', type = str, required = True,
        help='assign the port to load'
    )
    parser_open.add_argument(
        '-b', '--baudrate',
        dest='baudrate', action ='store', type = int,
        default=38400,
        help='assign the baudrate'
    )
    parser_open.add_argument(
        '-i', '--id',
        dest='id', action ='store', type = int,
        default=1,
        help='assign the terminal ID to open, available num is 1, 2 \n Default is 1.'
    )

    # parser for the "close" command
    parser_close = subparsers.add_parser('close', help='Close the terminal in adt.')
    parser_close.add_argument(
        '-i', '--id',
        dest='id', action ='store', type = int,
        default=1,
        help='assign the terminal ID to close, available num is 1, 2. \n Default is 1.'
    )

    # parser for the "close" command
    parser_clear = subparsers.add_parser('clear', help='Clear the terminal in adt.')
    parser_clear.add_argument(
        '-i', '--id',
        dest='id', action ='store', type = int,
        default=1,
        help='assign the terminal ID to clear, available num is 1, 2. \n Default is 1.'
    )
    args = parser.parse_args()
    return args

def termOpen(port, baudrate, id):
    adtsh = SocketHandler()
    cmd = {
        'cmd'    : AdtCmd.TERM.value,
        'subcmd' : AdtSubCmdTerm.OPEN.value,
        'port'     : port,
        'baudrate' : baudrate,
        'id'       : id
    }
    res = adtsh.send_cmd(cmd)

    if res['err']:
        print('Open port {} in terminal {} error.'.format(port, id))
        print('Error msg: {}'.format(res['msg']))

def termClose(id):
    adtsh = SocketHandler()
    cmd = {
        'cmd'    : AdtCmd.TERM.value,
        'subcmd' : AdtSubCmdTerm.CLOSE.value,
        'id'     : id
    }
    res = adtsh.send_cmd(cmd)

    if res['err']:
        print('Close terminal {} error.'.format(id))
        print('Error msg: {}'.format(res['msg']))

def termClear(id):
    adtsh = SocketHandler()
    cmd = {
        'cmd'    : AdtCmd.TERM.value,
        'subcmd' : AdtSubCmdTerm.CLEAR.value,
        'id'     : id
    }
    res = adtsh.send_cmd(cmd)

    if res['err']:
        print('Clear terminal {} error.'.format(id))
        print('Error msg: {}'.format(res['msg']))

def run():
    args = argHandler()
    if args.subcmd == 'open':
        termOpen(args.port, args.baudrate, args.id)
    elif args.subcmd == 'close':
        termClose(args.id)
    elif args.subcmd == 'clear':
        termClear(args.id)

if __name__ == '__main__':
    run()



from asa_hmi_data_agent.listport import getAvailableSerialPorts
from asa_hmi_data_agent.ui.ui_widget_asa_loader import Ui_WidgetAsaLoader
from asa_hmi_data_agent.util import ADTPATH

from PyQt5.QtCore import pyqtSlot, QThread, pyqtSignal, QObject
from PyQt5.QtWidgets import QFileDialog

from time import sleep
import datetime
import asaprog
import serial
import math
import os

# ---- class LoaderThread Start -------------------------------------------------
class LoaderThread(QThread):
    sigState = pyqtSignal(bool, int, str)
    sigChkAsaDevice = pyqtSignal(bool)
    sigStart = pyqtSignal(int)
    sigStepDone = pyqtSignal(int)
    sigEnd = pyqtSignal(bool)
    sigGetSerialException = pyqtSignal()

    def __init__(self):
        QThread.__init__(self)
        self.cmd = str()

    def setParameter(self, port, hexfile):
        self.port = port
        self.hexfile = hexfile
        self.max = 0

    def run(self):
        loader = asaprog.Loader(self.port, self.hexfile)
        self.loader = loader
        self.sigStart.emit(loader.total_steps-1)

        for i in range(loader.total_steps):
            try:
                loader.step()
            except asaprog.ChkDeviceException as e:
                self.sigChkAsaDevice.emit(False)
                debugLog('Error: The device is not asa-board.')
                break
            except asaprog.CantOpenComportException as e:
                self.sigGetSerialException.emit()
                debugLog('Error: Cannot open the comport \'{}\'.'.format(loader.port))
                break
            except asaprog.EndingException as e:
                self.sigEnd.emit(False)
                debugLog('Error: The device ignored the ending command .')
                break
            except serial.serialutil.SerialException as e:
                self.sigGetSerialException.emit()
                break

            if i == 0:
                self.sigStepDone.emit(i)
                self.sigChkAsaDevice.emit(True)
            elif i == loader.total_steps-1:
                self.sigStepDone.emit(i)
                self.sigEnd.emit(True)
            else:
                self.sigStepDone.emit(i)

    def stop(self):
        self.terminate()

# ---- class LoaderThread End ---------------------------------------------------
class AsaLoader(QObject):
    sigChangeWindowTitle = pyqtSignal(bool, str)
    sigSerialPortCheck = pyqtSignal(bool, str)

    # ---- __init__ start ------------------------------------------------------
    def __init__(self, widget):
        super(AsaLoader, self).__init__()
        self.ui = Ui_WidgetAsaLoader()
        self.ui.setupUi(widget)
        self.ui.progressBar.setValue(0)

        # ---- Thread Init start -----------------------------------------------
        self.loaderThread = LoaderThread()
        self.loaderThread.sigChkAsaDevice[bool].connect(self.updateStatus_isAsaDevice)
        self.loaderThread.sigStart[int].connect(self.initProgressbar)
        self.loaderThread.sigStepDone[int].connect(self.setProgressbar)
        self.loaderThread.sigEnd[bool].connect(self.updateStatus_end)
        self.loaderThread.sigGetSerialException.connect(self.updateStatus_serialException)
        self.loaderThread.finished.connect(
            lambda: self.sigSerialPortCheck.emit(
                False, self.ui.comboBox_selectPort.currentText()
            )
        )
        # ---- Thread Init end -------------------------------------------------

        # ---- Serial Group start ----------------------------------------------
        self.serial_updatePortlist()
        self.ui.pushButton_updatePortList.clicked.connect(self.serial_updatePortlist)
        # ---- Serial Group end ------------------------------------------------

        # ---- Basic Functions Group start -------------------------------------
        self.ui.pushButton_selectFile.clicked.connect(self.chooseProgFile)
        self.ui.pushButton_startProg.clicked.connect(self.startLoad)
        self.ui.pushButton_stopProg.clicked.connect(self.stopLoading)
        # ---- Basic Functions Group end ---------------------------------------

        # ---- Special Functions Group start -----------------------------------
        self.ui.pushButton_progStk500.clicked.connect(self.startLoadStk500)
        # ---- Special Functions Group end -------------------------------------

    # ---- LoaderThread Related functions start --------------------------------
    def initProgressbar(self, max):
        self.ui.progressBar.setRange(0, max)

    def setProgressbar(self, val):
        self.ui.progressBar.setValue(val)

    def updateStatus_isAsaDevice(self, isAsaDevice):
        if isAsaDevice:
            self.ui.label_statusContent.setText(u"確認裝置為ASA_M128，開始燒錄")
        else:
            self.ui.label_statusContent.setText(u"請確認連接裝置為ASA_M128，並按下reset按鈕")

    def updateStatus_end(self ,isOK):
        if isOK:
            self.ui.label_statusContent.setText(u"燒錄成功！")
        else:
            self.ui.label_statusContent.setText(u"燒錄失敗！")

    def updateStatus_serialException(self):
        self.ui.label_statusContent.setText(u"與串列埠失去連線，請檢察USB線是否有連接好")
    # ---- LoaderThread Related functions end ----------------------------------

    # ---- Serial Related functions start --------------------------------------
    def serial_updatePortlist(self):
        """Update port list in comboBox_selectPort."""
        availablePorts = getAvailableSerialPorts()
        debugLog('Update ports: ' + str(availablePorts))
        self.ui.comboBox_selectPort.clear()
        for port in availablePorts:
            self.ui.comboBox_selectPort.addItem(port)
    # ---- Serial Related functions end ----------------------------------------

    # ---- Basic Functions Group start -----------------------------------------
    def chooseProgFile(self):
        name, _ = QFileDialog.getOpenFileName(
            filter='All Files (*);;Hex Files (*.hex)',
            initialFilter='Hex Files (*.hex)'
        )
        if name is not '':
            self.ui.lineEdit_selectFile.setText(name)
            self.checkIsHexFile(name)

    def checkIsHexFile(self, filename):
        try:
            hexbin = asaprog.parseIHex(filename)
        except (NameError,UnicodeDecodeError):
            self.ui.label_programSizeContent.setText(u"非HEX檔案格式，請重新選擇檔案")
            self.ui.label_etcContent.setText('-')
            return False
        else:
            size = len(hexbin)/1024
            self.ui.label_programSizeContent.setText('{:0.2f} KB'.format(size))
            etc = math.ceil(len(hexbin)/256)*0.03
            self.ui.label_etcContent.setText('{:0.2f} s'.format(etc))
            return True

    def startLoad(self):
        if self.loaderThread.isRunning():
            return
        port = self.ui.comboBox_selectPort.currentText()
        if port == '':
            self.ui.label_statusContent.setText(u"未選擇串列埠")
            return
        hexfile = self.ui.lineEdit_selectFile.text()
        if hexfile == '':
            self.ui.label_statusContent.setText(u"未選擇燒錄檔案")
            return
        elif self.checkIsHexFile(hexfile) is False:
            self.ui.label_statusContent.setText(u"請重新選擇燒錄檔案")
            return

        self.sigSerialPortCheck.emit(True, port)
        self.ui.label_statusContent.setText(u"確認裝置中...")
        self.loaderThread.setParameter(port, hexfile)
        self.loaderThread.start()

    def stopLoading(self):
        if self.loaderThread.isRunning():
            self.loaderThread.stop()
            self.sigSerialPortCheck.emit(False, self.ui.comboBox_selectPort.currentText())
            self.ui.label_statusContent.setText(u"已強制終止")
    # ---- Basic Functions Group end -------------------------------------------

    # ---- Special Functions Group start ---------------------------------------
    def startLoadStk500(self):
        if self.loaderThread.isRunning():
            return
        port = self.ui.comboBox_selectPort.currentText()
        if port == '':
            self.ui.label_statusContent.setText(u"未選擇串列埠")
            return
        hexfile = os.path.join(ADTPATH, 'tools\m128_stk500.hex')
        self.checkIsHexFile(hexfile)

        self.sigSerialPortCheck.emit(True, port)
        self.ui.label_statusContent.setText(u"確認裝置中...")
        self.loaderThread.setParameter(port, hexfile)
        self.loaderThread.start()
    # ---- Special Functions Group end -----------------------------------------

def debugLog(msg):
    s = '[{}] loader   log: {}'.format(
        datetime.datetime.now().strftime("%H:%M:%S"),
        msg
    )
    print(s)
